package app;

import javafx.collections.FXCollections;
import javafx.geometry.Insets;
import javafx.geometry.Orientation;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.VBox;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.Region;

import javafx.scene.control.Label;
import javafx.scene.control.Separator;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.control.ChoiceBox;

import java.io.File;
import java.io.FileNotFoundException;
import java.nio.file.Paths;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Scanner;

public class Controller {
    public Estacionamiento estacionamiento = null;
    
    public BorderPane main_panel;

    // TOP
    public Label label_counter;

    // LEFT
    public VBox listadoAutos;

    // RIGHT
    public VBox formBox;
    public TextField field_patente;
    public ChoiceBox<String> field_tipo;

    // BOTTOM
    public Label messageLabel;


    public Controller( String configFile ) {

        // Configuramos el estacionamiento de acuerdo al archivo de configuración
        String filePath = Paths.get(System.getProperty("user.dir").toString(), configFile).toString();
        Scanner in = null;
        try {
            in = new Scanner(new File( filePath ));
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            System.exit(-1);
        }
        
        String capacidadesStr = in.nextLine();
        int[] capacidades = String2IntArr( capacidadesStr.split(" ") );
        
        String tarifasStr = in.nextLine();
        int[] tarifas = String2IntArr( tarifasStr.split(" ") );

        estacionamiento = new Estacionamiento( capacidades, tarifas );

        // Armamos la UI
        main_panel = new BorderPane();
        main_panel.setPadding(new Insets(10,10,10,10));

        // TOP
        VBox topContainer = new VBox(5);
        label_counter = new Label("0");
        topContainer.getChildren().addAll( label_counter, new Separator(Orientation.HORIZONTAL) );

        main_panel.setTop( topContainer );

        // CENTER
        listadoAutos = new VBox(5);
        listadoAutos.setPadding( new Insets(10, 10, 10, 0));

        main_panel.setCenter( listadoAutos );

        // RIGHT
        formBox = new VBox(10);
        formBox.setPadding( new Insets(10, 0, 10, 0));

        VBox textfieldBox = new VBox(5);
        field_patente = new TextField("");
        textfieldBox.getChildren().addAll( new Label("Patente:"), field_patente ); 
        formBox.getChildren().add( textfieldBox );

        field_tipo = new ChoiceBox<>( FXCollections.observableArrayList(new String[] { "General", "VIP", "Discapacitado" }));
        field_tipo.getSelectionModel().select(0);
        formBox.getChildren().add( field_tipo );

        // Agregamos un botón
        Button submitBtn = new Button("Agregar automóvil");
        submitBtn.setOnAction( event -> AgregarAutomovil() );
        formBox.getChildren().add( submitBtn );

        main_panel.setRight( formBox );


        // BOTTOM
        VBox bottomContainer = new VBox(5);
        messageLabel = new Label("");
        messageLabel.setId("messageDisplayer");
        messageLabel.setOnMousePressed( event -> messageLabel.setText("") );
        bottomContainer.getChildren().addAll( new Separator(Orientation.HORIZONTAL), messageLabel );
        main_panel.setBottom( bottomContainer );

        UpdateCounter();
    }


    public void AgregarAutomovil(){
        if ( field_patente.getText().isEmpty() || field_patente.getText().length() != 6 ){
            SetMessage("La patente debe tener 6 caracteres.");
            return;
        }
        
        Estacionamiento.Tipe tipo = Estacionamiento.Tipe.valueOf( field_tipo.getValue() );
        Automovil auto = new Automovil( field_patente.getText(), tipo );

        // Buscamos si el auto ya está dentro
        for ( int i = 0; i < estacionamiento.autos.size(); i++ ) if ( estacionamiento.autos.get(i).patente.equals( auto.patente ) ){
            SetMessage("Esa patente ya está dentro del estacionamiento.");
            return;
        }

        // Reviso y disminuye la disponibilidad
        if ( estacionamiento.GetAvailableSpace( tipo ) > 0 ){
            auto.espacioUtilizado = tipo;
            SetMessage("");
        } else if ( estacionamiento.GetAvailableSpace( Estacionamiento.Tipe.General ) > 0 ){
            auto.espacioUtilizado = Estacionamiento.Tipe.General;
            SetMessage("No hay espacio para el estacionamiento " + tipo + ", se envió al estacionamiento General.");
        } else {
            SetMessage("No hay más espacio en el estacionamiento.");
            return;
        }

        ResetForm();
        
        // Lo agregamos al listado
        estacionamiento.autos.add( auto );

        estacionamiento.SetCurrentCapacity(auto.espacioUtilizado, 1);
        AddCarToVBox( auto );
        UpdateCounter();
    }
    public void DespacharAutomovil( Automovil auto ){
        
        // Lo sacamos del listado del estacionamiento
        estacionamiento.SetCurrentCapacity(auto.espacioUtilizado, -1);
        estacionamiento.autos.remove( auto );
        RemoveCarFromVBox(auto);

        // Calcular la tarifa
        Duration duration = Duration.between(auto.init, LocalDateTime.now() );
        long minutes = Math.max( duration.toMinutesPart(), 30 );
        float total = estacionamiento.tarifas.get( auto.espacioUtilizado ) * minutes;

        String finalMessage = "Se despachó al auto de patente " + auto.patente;
        finalMessage += " | $" + estacionamiento.tarifas.get( auto.espacioUtilizado ) + " * " + minutes + "m = $" + total;

        SetMessage( finalMessage );
        UpdateCounter();
    }

    public void AddCarToVBox( Automovil auto ){
        HBox hbox = new HBox(15);
        hbox.setSpacing(10.0);
        auto.UI_Element = hbox;

        // Agregamos una etiqueta
        Label label = new Label(auto.patente + " - " + auto.espacioUtilizado);
        
        Region spacer = new Region();
        HBox.setHgrow(spacer, Priority.ALWAYS);

        // Agregamos un botón pa despachar
        Button dispatch_btn = new Button("Despachar");
        dispatch_btn.setOnAction( event -> {
            DespacharAutomovil( auto );
        });

        hbox.getChildren().addAll( label, spacer, dispatch_btn );
        listadoAutos.getChildren().add( hbox );
    }
    public void RemoveCarFromVBox( Automovil auto ){
        listadoAutos.getChildren().remove( auto.UI_Element );
    }


    public void SetMessage( String texto ){
        messageLabel.setText( texto );
    }
    public void ResetForm(){
        field_patente.setText("");
        field_tipo.setValue("General");
    }
    public void SetStyle( String filename ){
        main_panel.getStylesheets().clear();
        main_panel.getStylesheets().add( filename );
    }
    private void UpdateCounter(){
        String finalText = "General: " + estacionamiento.capacidadActual.get(Estacionamiento.Tipe.General) + "/" + estacionamiento.capacidad.get(Estacionamiento.Tipe.General);
        finalText += "\nVIP: " + estacionamiento.capacidadActual.get(Estacionamiento.Tipe.VIP) + "/" + estacionamiento.capacidad.get(Estacionamiento.Tipe.VIP);
        finalText += "\nDiscapacitados: " + estacionamiento.capacidadActual.get(Estacionamiento.Tipe.Discapacitado) + "/" + estacionamiento.capacidad.get(Estacionamiento.Tipe.Discapacitado);

        label_counter.setText( finalText );
    }

    private int[] String2IntArr( String[] arr ){
        int[] intArray = new int[arr.length];
        for (int i = 0; i < arr.length; i++) intArray[i] = Integer.parseInt(arr[i]);
        return intArray;
    }
}